@@ -23,6 +23,7 @@ AUTHOR:        kingson fiasco
 #include "qtermconfig.h"
 #include "qtermglobal.h"
 #include "schemedialog.h"
+#include "osdmessage.h"
 
 #include <QApplication>
 #include <QPainter>
@@ -68,6 +69,7 @@ Screen::Screen(QWidget *parent, Buffer *buffer, Param *param, BBS *bbs)
     m_inputContent = NULL;
     m_pASCIIFont = NULL;
     m_pGeneralFont = NULL;
+    m_pMessage = new PageViewMessage(this);
 
     setFocusPolicy(Qt::ClickFocus);
     setAttribute(Qt::WA_InputMethodEnabled, true);
@@ -113,6 +115,11 @@ Screen::Screen(QWidget *parent, Buffer *buffer, Param *param, BBS *bbs)
 
 }
 
+PageViewMessage * Screen::osd()
+{
+    return m_pMessage;
+}
+
 Screen::~Screen()
 {
     delete [] m_pBlinkLine;
@@ -283,8 +290,12 @@ void Screen::wheelEvent(QWheelEvent * we)
 {
     if (Global::instance()->m_pref.bWheel)
         QApplication::sendEvent(m_pWindow, we);
-    else
-        QApplication::sendEvent(m_scrollBar, we);
+    else {
+        int old_value = m_scrollBar->value();
+        int step = m_scrollBar->singleStep()*we->delta()/8/15;
+        m_scrollBar->setValue(old_value-step);
+        we->accept();
+    }
 }
 
 
@@ -310,6 +321,8 @@ void Screen::initFontMetrics()
 
     m_pASCIIFont->setStyleStrategy(Global::instance()->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
     m_pGeneralFont->setStyleStrategy(Global::instance()->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
+    getFontMetrics();
+    m_pMessage->setFont(*m_pGeneralFont);
 }
 
 void Screen::updateFont()
@@ -353,6 +366,7 @@ void Screen::updateFont()
         delete m_inputContent;
         m_inputContent = NULL;
     }
+    m_pMessage->setFont(*m_pGeneralFont);
 }
 
 void Screen::getFontMetrics()
@@ -393,6 +407,7 @@ void Screen::generalFontChanged(const QFont & font)
     m_pGeneralFont = new QFont(font);
     m_pASCIIFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
     m_pGeneralFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
+    m_pMessage->setFont(*m_pGeneralFont);
     QResizeEvent* re = new QResizeEvent(size(), size());
     resizeEvent(re);
 }
@@ -436,7 +451,7 @@ void Screen::setScheme()
     m_color[4]  = Qt::darkBlue;
     m_color[5]  = Qt::darkMagenta;
     m_color[6]  = Qt::darkCyan;
-    m_color[7]  = Qt::darkGray;
+    m_color[7]  = Qt::lightGray;
     m_color[8]  = Qt::gray;
     m_color[9]  = Qt::red;
     m_color[10] = Qt::green;
